/*
   NSSplitView.m
 
   Copyright (C) 2000 Free Software Foundation, Inc.
 
   Author:  Marko Mikulicic <marko@openself.org>
   Date: August 2001
 
   This file is part of the GNUstep GUI Library.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "NSSplitView+Autosave.h"

@interface NSSplitView (AutosaveExtensionPrivate)
- (void)_autosaveRestore;
@end

@implementation NSSplitView (AutosaveExtension)

static NSMutableDictionary *autosaveNames = nil;
static NSMutableDictionary *autosaveEnables = nil;

static NSString *NSSplitViewPositionsKey = @"NSSplitViewPositions";

- copyWithZone: (NSZone *)z
{
  return RETAIN(self);
}

- (void)setAutosaveName: (NSString *)name
{
  if(!autosaveNames)
    autosaveNames = [NSMutableDictionary new];

  [autosaveNames setObject: name forKey: self];
}

- (NSString *) autosaveName
{
  if(!autosaveNames)
    autosaveNames = [NSMutableDictionary new];

  return [autosaveNames objectForKey: self];
}

- (void)setAutosaveDividerPosition: (BOOL)flag
{
  BOOL oldflag = [self autosaveDividerPosition];

  if(!oldflag & flag ) {
    [self _autosaveRestore];

    [[NSNotificationCenter defaultCenter] addObserver: self
					  selector: @selector(_autosaveDidResize:)
					  name: NSSplitViewDidResizeSubviewsNotification
					  object: self];
  }

  [autosaveEnables setObject: [NSNumber numberWithBool: flag]
		   forKey: self];
}

- (BOOL)autosaveDividerPosition
{
  BOOL flag = NO;
  if(!autosaveEnables)
    autosaveEnables = [NSMutableDictionary new];

  if([[autosaveEnables objectForKey: self] boolValue]) // handle nil
    flag = YES;
  return flag;
}

- (void)_autosaveDidResize: (id)not
{
  float fact = 0.0;
  NSRect r1,r2;

  id subviews = [self subviews];

  if(![self autosaveName] || ![self autosaveDividerPosition])
    return;

  if([subviews count] != 2) {
    [NSException raise: @"NSInternalInconsistency" format: @"wrong number of subviews (%d",
		 [subviews count]];
  }
  
  r1 = [[subviews objectAtIndex: 0] frame];
  r2 = [[subviews objectAtIndex: 1] frame];

  if([self isVertical]) 
    fact = r1.size.width / r2.size.width;
  else
    fact = r1.size.height / r2.size.height;

  {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSMutableDictionary *splitterConfig = [[defaults objectForKey: NSSplitViewPositionsKey] mutableCopy];
    NSNumber *thisConfig;

    if(splitterConfig == nil)
      splitterConfig = [NSMutableDictionary new];
    
    thisConfig = [NSNumber numberWithFloat: fact];
    [splitterConfig setObject: thisConfig forKey: [self autosaveName]];

    [defaults setObject: splitterConfig forKey: NSSplitViewPositionsKey];
    [defaults synchronize];
  }
}

- (void)_autosaveRestore
{
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  NSDictionary *splitterConfig = [defaults objectForKey: NSSplitViewPositionsKey];
  NSNumber *thisConfig;
  float ratio;
  NSView *v1,*v2;
  NSRect r1,r2;

  if(!splitterConfig) {
    return;
  }

  thisConfig = [splitterConfig objectForKey: [self autosaveName]];
  if(!thisConfig) {
    return;
  }

  ratio = [thisConfig floatValue];

  v1 = [[self subviews] objectAtIndex: 0];
  v2 = [[self subviews] objectAtIndex: 1];
  r1 = [v1 frame];
  r2 = [v2 frame];

  if([self isVertical]) {
    r1.size.width = ratio * r2.size.width;
    r2.size.width = r1.size.width / ratio;
  } else {
    r1.size.height = ratio * r2.size.height;
    r2.size.height = r1.size.height / ratio;
  }
  
  [v1 setFrame: r1];
  [v2 setFrame: r2];
}

@end
